<?php
class Update {	
	private $lLastUpdateCheck = 'restream.version.txt';
	private $lUpdateCheckUrl = 'http://theyosh.nl/restream.version.txt';
	private $lUpdateTimeOut = 86400; // 24 hours
	private $lUpdateFolder = 'update';
	
	private $lUpdateDownloadUrl;
	private $lUpdateVersion;	
	private $lUpdateHash;	
	private $lUpdateFile;
	private $lUpdateSourceLocation;	

	function __construct() {
		$this->lUpdateVersion = -1;
		$lCheckOnline = true;
		$lUpdateData = '';
		
		if (file_exists($this->lLastUpdateCheck)) {
			// Check chage age
			$lLastCheck = filemtime($this->lLastUpdateCheck);
			$lCheckOnline = ((time() - $lLastCheck) > $this->lUpdateTimeOut);
			$lUpdateData = file_get_contents($this->lLastUpdateCheck);
		}

		if ($lCheckOnline) {
			if (($lUpdateData = @file_get_contents($this->lUpdateCheckUrl)) === false) {
				$lUpdateData = '';
			}
		}
		if ($lUpdateData != '') {
			$lUpdateData = explode("\n",$lUpdateData);
			$this->lUpdateVersion = explode(" ",trim($lUpdateData[0]));
			$this->lUpdateDownloadUrl = trim($this->lUpdateVersion[1]);
			$this->lUpdateHash = trim($this->lUpdateVersion[2]);
			$this->lUpdateVersion = trim($this->lUpdateVersion[0]);
			$this->lUpdateFile = basename(parse_url($this->lUpdateDownloadUrl, PHP_URL_PATH));
			file_put_contents($this->lLastUpdateCheck, implode("\n",$lUpdateData));
		}
	}

	private function downloadUpdate() {
		$lReturnErrors = array();
		Utils::execute('rm -Rf ' . getcwd() . '/' . $this->lUpdateFolder,true);
		if (mkdir(getcwd() . '/' . $this->lUpdateFolder)) {
			$this->lUpdateFile = getcwd() . '/' . $this->lUpdateFolder . '/' . $this->lUpdateFile;
			if (($lUpdateZip = file_get_contents($this->lUpdateDownloadUrl)) !== false) {
				if (file_put_contents($this->lUpdateFile,$lUpdateZip) !== false) {
					if (sha1_file($this->lUpdateFile) == $this->lUpdateHash) {
						return true;
					} else {
						$lReturnErrors[] = 'Sha1 Hash is invalid.';
					}
				} else {
					$lReturnErrors[] = 'Can\'t write file do disk: ' . $this->lUpdateFile;
				}
			} else {
				$lReturnErrors[] = 'Can\'t download the update file from source: ' . $his->lUpdateDownloadUrl;
			}
		} else {
			$lReturnErrors[] = 'Can\'t create update folder. Make sure that the restream folder is writable by the web server. Or remove the existing update directory';
		}
		return $lReturnErrors;
	}
	
	private function unpackUpdate() {
		$lCMD = 'unzip ' . $this->lUpdateFile . ' -d ' . getcwd() . '/' . $this->lUpdateFolder;
		Utils::execute($lCMD,true);
		$lFolderData = scandir(getcwd() . '/' . $this->lUpdateFolder);
		foreach ($lFolderData as $lEntry) {
			if (substr($lEntry,0,1) == '.') continue;
			if (stripos($lEntry, 'restream') !== false && is_dir(getcwd() .'/' . $this->lUpdateFolder . '/' . $lEntry)) {
				$this->lUpdateSourceLocation = array_merge(array(getcwd() .'/' . $this->lUpdateFolder . '/' . $lEntry),$this->glob_recursive(getcwd() . '/' . $this->lUpdateFolder . '/' . $lEntry . '/*'));
				unlink($this->lUpdateFile);
				return true;
			}
		}
		return false;
	}

	private function glob_recursive($pattern, $flags = 0) {
	    $files = glob($pattern, $flags);   
	    foreach (glob(dirname($pattern).'/*', GLOB_ONLYDIR|GLOB_NOSORT) as $dir)
	    {
			$files = array_merge($files, $this->glob_recursive($dir.'/'.basename($pattern), $flags));
	    }
	    return $files;
	}

	private function checkWriteRights() {
		$lErrors = array();
		$lPrefixLength = strlen($this->lUpdateSourceLocation[0] . '/');
		if (!is_writable(getcwd())) $lErrors[] = getcwd();
		
		foreach ($this->lUpdateSourceLocation as $lUpdateFile) {
			$lFile = substr($lUpdateFile,$lPrefixLength);
			clearstatcache();
			if (file_exists($lFile)) {
				clearstatcache();
				if (!is_writable($lFile)) {
					clearstatcache();
					$lErrors[] = $lFile . ' is not writable by the webserver.';
				}
			} else {
				// Check if parent folder is writable...
				$lData = pathinfo($lUpdateFile);
			}		
		}
		return (count($lErrors) == 0 ? true : $lErrors);
	}
	
	private function moveUpdateFiles() {
		$lCMD = 'mv ' . $this->lUpdateSourceLocation[0] . '/* ' . getcwd() . '/ && rm -Rf ' . $this->lUpdateSourceLocation[0];
		Utils::execute($lCMD);
		return true;
	}
	
	public function updateAvailable() {
		return $this->lUpdateVersion > VERSION;
	}
	
	public function getLatestUpdateVersion() {
		return $this->lUpdateVersion;
	}
	public function runUpdate() {
		$lErrors = array();
		if ($this->updateAvailable()) {
			if ( ($lErrors = $this->downloadUpdate()) === true) {
				if ( ($lErrors = $this->unpackUpdate()) === true) {
					if (($lErrors = $this->checkWriteRights()) === true) {
						if (($lErrors = $this->moveUpdateFiles()) === true) {
							// reload the page....
							$lErrors = array();
						}
					}
				}
			}
		}
		$lAjaxResponse  = new xajaxResponse();
		if (count($lErrors) > 0) {
			$lAjaxResponse->assign("message","innerHTML","<p class=\"error\" style=\"font-weight: normal\">Errors during updating:<br />" . implode('<br />',$lErrors) . "</p>");
		} else {
			$lAjaxResponse->assign("message","innerHTML","<p>The updated succeded!. The page will reload in 10 seconds for the setup.</p>");
			$lAjaxResponse->script("setTimeout(function() {location.reload();},10000);");
		}
		return $lAjaxResponse;	
	}
}
